from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="anonLLM",
    version="0.1.8",
    author="FS Ndzomga",
    author_email="ndzomgafs@gmail.com",
    description="Anonymizes personally identifiable information for "
    "Large Language Model APIs",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/fsndzomga/anonLLM",
    project_urls={
        "Bug Tracker": "https://github.com/fsndzomga/anonLLM/issues"
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=find_packages(),
    python_requires=">=3.6",
    install_requires=[
        "aiohttp>=3.8,<3.9",
        "aiosignal>=1.3,<1.4",
        "annotated-types>=0.5,<0.6",
        "async-timeout>=4.0,<4.1",
        "attrs>=23.1,<24.0",
        "blis>=0.7,<0.8",
        "catalogue>=2.0,<2.1",
        "certifi>=2023.7,<2024.0",
        "charset-normalizer>=3.2,<3.3",
        "click>=8.1,<8.2",
        "confection>=0.1,<0.2",
        "cymem>=2.0,<2.1",
        "Faker>=19.5,<20.0",
        "frozenlist>=1.4,<1.5",
        "idna>=3.4,<3.5",
        "Jinja2>=3.1,<3.2",
        "langcodes>=3.3,<3.4",
        "MarkupSafe>=2.1,<2.2",
        "multidict>=6.0,<6.1",
        "murmurhash>=1.0,<1.1",
        "numpy>=1.25,<1.26",
        "openai>=0.28,<0.29",
        "packaging>=23.1,<24.0",
        "pathy>=0.10,<0.11",
        "pip>=22.3,<22.4",
        "preshed>=3.0,<3.1",
        "pydantic>=2.3,<2.4",
        "pydantic_core>=2.6,<2.7",
        "python-dateutil>=2.8,<2.9",
        "python-dotenv>=1.0,<1.1",
        "regex>=2023.8,<2023.9",
        "requests>=2.31,<2.32",
        "setuptools>=65.5,<65.6",
        "six>=1.16,<1.17",
        "smart-open>=6.4,<6.5",
        "spacy>=3.6,<3.7",
        "spacy-legacy>=3.0,<3.1",
        "spacy-loggers>=1.0,<1.1",
        "srsly>=2.4,<2.5",
        "thinc>=8.1,<8.2",
        "tiktoken>=0.4,<0.5",
        "tqdm>=4.66,<4.67",
        "typer>=0.9,<1.0",
        "typing_extensions>=4.7,<4.8",
        "urllib3>=2.0,<2.1",
        "wasabi>=1.1,<1.2",
        "yarl>=1.9,<1.10"
    ],
    extras_require={
        "dev": ["pytest", "flake8"],
    },
)
